/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alipay.sofa.jraft.rhea.benchmark.raw;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.rocksdb.HistogramData;
import org.rocksdb.HistogramType;
import org.rocksdb.StatisticsCollectorCallback;
import org.rocksdb.TickerType;

import com.alipay.sofa.jraft.rhea.options.RocksDBOptions;
import com.alipay.sofa.jraft.rhea.storage.RocksRawKVStore;

public class BaseRawStoreBenchmark {

    protected String          tempPath;
    protected RocksRawKVStore kvStore;
    protected RocksDBOptions  dbOptions;

    protected void setup() throws Exception {
        File file = getTempDir();
        this.tempPath = file.getAbsolutePath();
        System.out.println(this.tempPath);
        this.kvStore = new RocksRawKVStore();
        this.dbOptions = new RocksDBOptions();
        this.dbOptions.setDbPath(this.tempPath);
        this.dbOptions.setSync(false);
        this.dbOptions.setStatisticsCallbackIntervalSeconds(10);
        this.kvStore.init(this.dbOptions);
        StatisticsCollectorCallback callback = new StatisticsCollectorCallback() {

            @Override
            public void tickerCallback(TickerType c, long tickerCount) {
                System.out.print(c + " ");
                System.out.println(tickerCount);
            }

            @Override
            public void histogramCallback(HistogramType histType, HistogramData histData) {
                System.out.print(histType + " ");
                System.out.println(histData.getAverage());
            }
        };
        this.kvStore.addStatisticsCollectorCallback(callback);
    }

    protected File getTempDir() throws IOException {
        final File file = File.createTempFile("RawRocksDBTest", "test");
        FileUtils.forceDelete(file);
        if (file.mkdirs()) {
            return file;
        }
        throw new RuntimeException("fail to make dirs");
    }

    protected void tearDown() throws Exception {
        this.kvStore.shutdown();
        FileUtils.forceDelete(new File(this.tempPath));
    }
}
